;; s.t.o.3/theme.jl

;; 28 March 2000
;; by Nicholas Riley - comments to <sawmill@sabi.net>
;; based on the s.t.o Sawmill theme, by Michele Campeotto <golia@supereva.it>

;; changes:
;; - a readable color for non-focused window titles
;; - close box on the left, collapse (windowshade) box on the right
;; - non-hardcoded font setting
;; - border permitting resizing, and (optionally) resizing corners
;; - easily user-adjustable colors/widths of many of the above features

(let*
    (;; -- user configurable settings --

     ;; color for window border
     (border-color "#01f001f001f0")

     ;; width of window border
     (border-width 2)
     
     ;; colors for window resizing corner (disabled, enabled)
     (corner-color (list "#6d6d6d6d6d6d" "#52527272a3a3"))

     ;; length/width of sizing corner (added to border width)
     (corner-side 10)

     ;; border width of sizing corner (2 and 4 are good values)
     ;; (will protrude from window if > border-width)
     ;; set to 0 to remove sizing corner
     (corner-border-width 4)

     ;; offset of close box (may want to change with font)
     (closebox-offset 12)

     (title-width
      (lambda (w)
        (let
          ((w-width (car (window-dimensions w))))
        (max 0 (+ 20 (min (- w-width 55) (text-width (window-name w))))))))
     
     (make-border
      (lambda (class)
	(let ((tb (eq class 'top-border)) (bb (eq class 'bottom-border))
	      (lb (eq class 'left-border)) (rb (eq class 'right-border))
	      (corner-side (if (or (< corner-side 1) (< corner-border-width 1))
			       (- border-width) corner-side)))
	  (list `(background . ,border-color)
		(cond ((or tb bb) `(left-edge . ,corner-side))
		      ((or lb rb) `(top-edge . ,(+ -25 (+ border-width corner-side)))))
		`(,(if (or tb bb) 'height 'width) . ,border-width)
		(cond ((or tb bb) `(right-edge . ,corner-side))
		      (lb `(left-edge  . ,(- border-width)))
		      (rb `(right-edge . ,(- border-width))))
		(cond (tb `(top-edge . -25))
		      (bb `(bottom-edge . ,(- border-width)))
		      ((or lb rb) `(bottom-edge . ,corner-side)))
		`(class . ,class)))))

     (make-corner-horiz
      (lambda (class)
	(let ((tl (eq class 'top-left-corner)) (bl (eq class 'bottom-left-corner))
	      (tr (eq class 'top-right-corner)) (br (eq class 'bottom-right-corner)))
	  (list `(background . ,corner-color)
		(cond ((or tl bl) `(left-edge . ,(- corner-border-width)))
		      ((or tr br) `(right-edge . ,(- corner-border-width))))
		`(width . ,(+ corner-side corner-border-width))
		`(height . ,corner-border-width)
		(cond ((or tl tr) `(top-edge . ,(+ -25 (- border-width corner-border-width))))
		      ((or bl br) `(bottom-edge . ,(- corner-border-width))))
		`(class . ,class)))))

     (make-corner-vert
      (lambda (class)
	(let ((tl (eq class 'top-left-corner)) (bl (eq class 'bottom-left-corner))
	      (tr (eq class 'top-right-corner)) (br (eq class 'bottom-right-corner)))
	  (list `(background . ,corner-color)
		(cond ((or tl tr) `(top-edge . ,(+ -25 (- border-width corner-border-width))))
		      ((or bl br) `(bottom-edge . ,(- corner-border-width))))
		`(height . ,(+ corner-side corner-border-width))
		`(width . ,corner-border-width)
		(cond ((or tl bl) `(left-edge . ,(- corner-border-width)))
		      ((or tr br) `(right-edge . ,(- corner-border-width))))
		`(class . ,class)))))

     (titlebar-left (list (make-image "tl-i.png") (make-image "tl.png")))

     (titlebar-right (list (make-image "tr-i.png") (make-image "tr.png")))

     (title-colors (list "LightSteelBlue1" "white"))
     
     (middle (list (make-image "middle-i.png")
                   (make-image "middle.png")))
     
     (corner (list (make-image "corner-i.png")
                         (make-image "corner.png")))

     (close (list (make-image "close.gif")
                  (make-image "close.gif") nil
                  (make-image "close.gif")))

     (maximize (list (make-image "up-i.png")
		     (make-image "up.png") nil
                     (make-image "up.png")))

     (iconify (list (make-image "down-i.png")
			(make-image "down.png") nil
                    (make-image "down.png")))

     (shade (list (make-image "shade-i.png")
                  (make-image "shade.png") nil
                  (make-image "shade.png")))

     (menu (list (make-image "right-i.png")
                 (make-image "right.png") nil
                 (make-image "right.png")))

     ;; frame layout
     (frame `(
	      ;; titlebar (close button background)
              ((background . ,titlebar-left)
               (top-edge . -25)
               (left-edge . 0)
	       (width . 17)
               (cursor . left_ptr)
               (class . title))

	      ;; titlebar (title area)
	      ((background . ,titlebar-left)
	       (foreground . ,title-colors)
	       (text . ,window-name)
	       (x-justify . center)
	       (y-justify . center)
	       (top-edge . -25)
	       (left-edge . ,closebox-offset)
	       (width . ,(lambda (w) (title-width w)))
               (cursor . left_ptr)
	       (class . title))

	      ;; titlebar (middle)
	      ((background . ,middle)
	       (left-edge . ,(lambda (w) (+ closebox-offset (title-width w))))
	       (top-edge . -25)
               (cursor . left_ptr)
	       (class . title))

	      ;; titlebar (right side)
	      ((background . ,titlebar-right)
	       (top-edge . -25)
	       (left-edge . ,(lambda (w) (+ (title-width w) closebox-offset 25)))
	       (right-edge . 0)
               (cursor . left_ptr)
	       (class . title))

              ;; close button
              ((background . ,close)
	       (foreground . ,title-colors)
               (top-edge . -25)
               (left-edge . 3)
               (cursor . left_ptr)
               (class . close-button))

              ;; maximize button
              ((background . ,maximize)
               (top-edge . -25)
               (right-edge . 20)
               (cursor . left_ptr)
               (class . maximize-button))

              ;; iconify button
              ((background . ,iconify)
               (top-edge . -25)
               (right-edge . 37)
               (cursor . left_ptr)
               (class . iconify-button))

              ;; shade button
              ((background . ,shade)
               (top-edge . -25)
               (right-edge . 2)
               (cursor . left_ptr)
               (class . shade-button))

              ;; resize corner
              ((background . ,corner)
               (bottom-edge . 0)
               (right-edge . 0)
               (cursor . left_ptr)
               (class . bottom-right-corner))

 	      ;; top border
	      ,(make-border 'top-border)

 	      ;; left border
	      ,(make-border 'left-border)

	      ;; right border
	      ,(make-border 'right-border)

	      ;; bottom border
	      ,(make-border 'bottom-border)

 	      ;; top left corner
	      ,(make-corner-horiz 'top-left-corner)
	      ,(make-corner-vert 'top-left-corner)

	      ;; top right corner
	      ,(make-corner-horiz 'top-right-corner)
	      ,(make-corner-vert 'top-right-corner)

	      ;; bottom left corner
	      ,(make-corner-horiz 'bottom-left-corner)
	      ,(make-corner-vert 'bottom-left-corner)
	      
	      ;; bottom right corner
	      ,(make-corner-horiz 'bottom-right-corner)
	      ,(make-corner-vert 'bottom-right-corner)

       ))

       (shaped-frame `(
	      ;; titlebar (left side)
	      ((background . ,titlebar-left)
	       (foreground . ,title-colors)
	       (text . ,window-name)
	       (x-justify . center)
	       (y-justify . center)
	       (top-edge . -25)
	       (left-edge . 0)
	       (width . ,(lambda (w) (title-width w)))
               (cursor . left_ptr)
	       (class . title))

	      ;; middle
	      ((background . ,middle)
	       (left-edge . ,(lambda (w) (title-width w)))
	       (top-edge . -25)
               (cursor . left_ptr)
	       (class . title))

	      ;; titlebar (right side)
	      ((background . ,titlebar-right)
	       (top-edge . -25)
	       (left-edge . ,(lambda (w) (+ (title-width w) 25)))
	       (right-edge . 0)
               (cursor . left_ptr)
	       (class . title))

              ;; shade button
              ((background . ,shade)
               (top-edge . -25)
               (right-edge . 0)
               (cursor . left_ptr)
               (class . shade-button))
       ))

       (transient-frame `(
	      ;; titlebar (left side)
	      ((background . ,titlebar-left)
	       (foreground . ,title-colors)
	       (text . ,window-name)
	       (x-justify . center)
	       (y-justify . center)
	       (top-edge . -25)
	       (left-edge . 0)
	       (width . ,(lambda (w) (title-width w)))
               (cursor . left_ptr)
	       (class . title))

	      ;; middle
	      ((background . ,middle)
	       (left-edge . ,(lambda (w) (title-width w)))
	       (top-edge . -25)
               (cursor . left_ptr)
	       (class . title))

	      ;; titlebar (right side)
	      ((background . ,titlebar-right)
	       (top-edge . -25)
	       (left-edge . ,(lambda (w) (+ (title-width w) 25)))
	       (right-edge . 0)
               (cursor . left_ptr)
	       (class . title))

              ;; shade button
              ((background . ,shade)
               (top-edge . -25)
               (right-edge . 0)
               (cursor . left_ptr)
               (class . shade-button))
       ))

       (shaped-transient-frame `(
	      ;; titlebar (left side)
	      ((background . ,titlebar-left)
	       (foreground . ,title-colors)
	       (text . ,window-name)
	       (x-justify . center)
	       (y-justify . center)
	       (top-edge . -25)
	       (left-edge . 0)
	       (width . ,(lambda (w) (title-width w)))
               (cursor . left_ptr)
	       (class . title))

	      ;; middle
	      ((background . ,middle)
	       (left-edge . ,(lambda (w) (title-width w)))
	       (top-edge . -25)
               (cursor . left_ptr)
	       (class . title))

	      ;; titlebar (right side)
	      ((background . ,titlebar-right)
	       (top-edge . -25)
	       (left-edge . ,(lambda (w) (+ (title-width w) 25)))
	       (right-edge . 0)
               (cursor . left_ptr)
	       (class . title))

              ;; shade button
              ((background . ,shade)
               (top-edge . -25)
               (right-edge . 0)
               (cursor . left_ptr)
               (class . shade-button))
       )))
  
  (def-frame-class shade-button '((cursor . left_ptr))
      (bind-keys shade-button-keymap "Button1-Off" 'toggle-window-shaded)
      (bind-keys shade-button-keymap "Button3-Click1" 'popup-window-menu))

  (add-frame-style 's.t.o.3
		   (lambda (w type)
		     (cond ((eq type 'shaped)
			    shaped-frame)
			   ((eq type 'transient)
			    transient-frame)
			   ((eq type 'shaped-transient)
			    shaped-transient-frame)
			   ((eq type 'unframed)
			    nil-frame)
			   (t
			    frame))))

  (call-after-property-changed 'WM_NAME
    (lambda ()
      (rebuild-frames-with-style 's.t.o.3)))
)
